import nltk
import sys

TERMINALS = """
Adj -> "country" | "dreadful" | "enigmatical" | "little" | "moist" | "red"
Adv -> "down" | "here" | "never"
Conj -> "and" | "until"
Det -> "a" | "an" | "his" | "my" | "the"
N -> "armchair" | "companion" | "day" | "door" | "hand" | "he" | "himself"
N -> "holmes" | "home" | "i" | "mess" | "paint" | "palm" | "pipe" | "she"
N -> "smile" | "thursday" | "walk" | "we" | "word"
P -> "at" | "before" | "in" | "of" | "on" | "to"
V -> "arrived" | "came" | "chuckled" | "had" | "lit" | "said" | "sat"
V -> "smiled" | "tell" | "were"
"""

NONTERMINALS = """
S -> NNP VP | NNP VP NNP | S Conj S | PRE VP Conj S Adv | S NS | S NSE | S NNP
NNP -> P NP | NP
PRE -> NP A 
NP -> N | Det N | Det A N 
VP -> V | V NP | V NP V | V NP V NP | V NP Conj | V A
NS -> Conj V NP
NSE -> NS NNP
A -> Adj | Adj A | Adv
"""

grammar = nltk.CFG.fromstring(NONTERMINALS + TERMINALS)
parser = nltk.ChartParser(grammar)


def main():

    # If filename specified, read sentence from file
    if len(sys.argv) == 2:
        with open(sys.argv[1]) as f:
            s = f.read()

    # Otherwise, get sentence as input
    else:
        s = input("Sentence: ")

    # Convert input into list of words
    s = preprocess(s)

    # Attempt to parse sentence
    try:
        trees = list(parser.parse(s))
    except ValueError as e:
        print(e)
        return
    if not trees:
        print("Could not parse sentence.")
        return

    # Print each tree with noun phrase chunks
    for tree in trees:
        tree.pretty_print()

        print("Noun Phrase Chunks")
        for np in np_chunk(tree):
            print(" ".join(np.flatten()))


def preprocess(sentence):
    """
    Convert `sentence` to a list of its words.
    Pre-process sentence by converting all characters to lowercase
    and removing any word that does not contain at least one alphabetic
    character.
    """

    # create tokens
    tokens = nltk.word_tokenize(sentence)

    # create a return list
    processed = []

    # iterate over words
    for word in tokens:
        # check for alphabetical characters
        alphabetical = 0
        for character in word:
            if character.isalpha():
                alphabetical += 1
        # if there are some, then add the lowercase word to the final list
        if alphabetical > 0:
            processed.append(word.lower())

    return processed


def np_chunk(tree):
    """
    Return a list of all noun phrase chunks in the sentence tree.
    A noun phrase chunk is defined as any subtree of the sentence
    whose label is "NP" that does not itself contain any other
    noun phrases as subtrees.
    """

    # list to return
    chunk = []

    # iterate over all subtrees in the tree
    for s in tree.subtrees():
        if s.label() == 'NP':
            counter = 0
            # for all NP labels, check their subtrees
            for ss in s.subtrees():
                if ss.label() == 'NP':
                    # count how many subtrees have the label NP
                    counter += 1

            # counter should be 1 if this is a final NP, since it counts itself
            if counter == 1:
                chunk.append(s)

    return chunk


if __name__ == "__main__":
    main()
